"use client";

import React, { useEffect, useState, Suspense } from 'react';
import { FaStar } from 'react-icons/fa';
import Link from 'next/link';
import { generateSlug } from "@/lib/generateSlug";
import { usePathname, useSearchParams, useRouter } from 'next/navigation';
import Head from 'next/head';
import AdSense from '@/app/components/AdSense';


interface Movie {
  id: number;
  title: string;
  poster_path: string;
  vote_average: number;
  release_date: string;
}

interface Genre {
  id: number;
  name: string;
}

const Movies: React.FC = () => {
  const [latestMovies, setLatestMovies] = useState<Movie[]>([]);
  const [itemsToShow, setItemsToShow] = useState(40);
  const [fadeKey, setFadeKey] = useState(0);
  const [selectedGenre, setSelectedGenre] = useState<number | null>(null);
  const [selectedYear, setSelectedYear] = useState<string | null>(null);
  const [genres, setGenres] = useState<Genre[]>([]);
  const [years, setYears] = useState<string[]>([]);
  const [currentPage, setCurrentPage] = useState(1);
  const [totalPages, setTotalPages] = useState(1);

  const pathname = usePathname();
  const searchParams = useSearchParams();
  const router = useRouter();

  useEffect(() => {
    const calculateItemsToShow = () => {
      const screenWidth = window.innerWidth;
      if (screenWidth >= 1024) {
        setItemsToShow(40); // 40 items for PC
      } else {
        setItemsToShow(21); // 21 items for mobile
      }
    };

    calculateItemsToShow();
    window.addEventListener('resize', calculateItemsToShow);

    return () => {
      window.removeEventListener('resize', calculateItemsToShow);
    };
  }, []);

  useEffect(() => {
    const fetchGenres = async () => {
      try {
        const response = await fetch(
          `https://api.themoviedb.org/3/genre/movie/list?api_key=${process.env.NEXT_PUBLIC_TMDB_API_KEY}&language=en-US`
        );
        const data = await response.json();
        setGenres(data.genres);
      } catch (error) {
        console.error('Failed to fetch genres:', error);
      }
    };

    const generateYears = () => {
      const currentYear = new Date().getFullYear();
      const availableYears = [];
      for (let year = currentYear; year >= 1999; year--) {
        availableYears.push(year.toString());
      }
      setYears(availableYears);
    };

    fetchGenres();
    generateYears();
  }, []);

  useEffect(() => {
    const fetchTrendingMovies = async (page: number = 1) => {
      try {
        const response = await fetch(
          `https://api.themoviedb.org/3/trending/movie/day?api_key=${process.env.NEXT_PUBLIC_TMDB_API_KEY}&page=${page}`
        );
        const data = await response.json();

        setTotalPages(data.total_pages);

        return data.results;
      } catch (error) {
        console.error('Failed to fetch trending movies:', error);
        return [];
      }
    };

    const fetchMovies = async (page: number = 1) => {
      let url = `https://api.themoviedb.org/3/discover/movie?api_key=${process.env.NEXT_PUBLIC_TMDB_API_KEY}&page=${page}&sort_by=release_date.desc&vote_count.gte=1`;

      if (selectedGenre) {
        url += `&with_genres=${selectedGenre}`;
      }
      if (selectedYear) {
        url += `&primary_release_year=${selectedYear}`;
      }

      try {
        const response = await fetch(url);
        const data = await response.json();

        setTotalPages(data.total_pages);

        const filteredMovies = data.results.filter(
          (movie: Movie) =>
            new Date(movie.release_date).getTime() <= new Date().getTime() && movie.vote_average > 0
        );

        return filteredMovies;
      } catch (error) {
        console.error('Failed to fetch movies:', error);
        return [];
      }
    };

    const loadMovies = async () => {
      let movies = [];
      if (selectedGenre || selectedYear) {
        const page1Movies = await fetchMovies(currentPage);
        const page2Movies = itemsToShow > 20 ? await fetchMovies(currentPage + 1) : [];
        movies = [...page1Movies, ...page2Movies];
      } else {
        const page1Movies = await fetchTrendingMovies(currentPage);
        const page2Movies = itemsToShow > 20 ? await fetchTrendingMovies(currentPage + 1) : [];
        movies = [...page1Movies, ...page2Movies];
      }

      setLatestMovies(movies.slice(0, itemsToShow));
      setFadeKey((prevKey) => prevKey + 1);
    };

    loadMovies();
  }, [itemsToShow, selectedGenre, selectedYear, currentPage]);

  useEffect(() => {
    const params = new URLSearchParams();
    if (selectedGenre) {
      params.set('genre', selectedGenre.toString());
    }
    if (selectedYear) {
      params.set('year', selectedYear);
    }
    router.replace(`${pathname}?${params.toString()}`);
  }, [selectedGenre, selectedYear, pathname, router]);

  const renderPagination = () => {
    const pages = [];
    const startPage = Math.max(1, currentPage - 2);
    const endPage = Math.min(totalPages, startPage + 4);

    for (let i = startPage; i <= endPage; i++) {
      pages.push(
        <button
          key={i}
          onClick={() => setCurrentPage(i)}
          className={`mx-1 px-3 py-1 rounded ${i === currentPage ? 'bg-red-600 text-white' : 'bg-gray-800 text-gray-300'}`}
        >
          {i}
        </button>
      );
    }

    return (
      <div className="flex justify-center mt-6">
        <button
          onClick={() => setCurrentPage(currentPage - 1)}
          className="mx-1 px-3 py-1 rounded bg-gray-800 text-gray-300"
          disabled={currentPage === 1}
        >
          &laquo; Previous
        </button>
        {pages}
        <button
          onClick={() => setCurrentPage(currentPage + 1)}
          className="mx-1 px-3 py-1 rounded bg-gray-800 text-gray-300"
          disabled={currentPage === totalPages}
        >
          Next &raquo;
        </button>
      </div>
    );
  };

  return (
    <>
      <Head>
        <title>Delgor - Explore Movies</title>
        <meta name="description" content="Browse and discover the latest movies on Delgor. Find your favorite genres and years, and enjoy a seamless movie-watching experience." />
        <link rel="canonical" href="https://www.delgor.com/movies" />
        <meta property="og:title" content="Delgor - Explore Movies" />
        <meta property="og:description" content="Browse and discover the latest movies on Delgor. Find your favorite genres and years, and enjoy a seamless movie-watching experience." />
        <meta property="og:type" content="website" />
        <meta property="og:url" content="https://www.delgor.com/movies" />
        <meta property="og:image" content="/images/ogdelgor.png" /> {/* Replace with the actual OG image */}
        <meta property="og:image:alt" content="Delgor Movies" />
        <meta property="og:site_name" content="Delgor" />
        {/* Structured Data for the Movies Page */}
        <script type="application/ld+json">
          {JSON.stringify({
            "@context": "https://schema.org",
            "@type": "Movie",
            "name": "Delgor - Explore Movies",
            "description": "Browse and discover the latest movies on Delgor. Find your favorite genres and years, and enjoy a seamless movie-watching experience.",
            "url": "https://www.delgor.com/movies",
            "image": "/images/ogdelgor.png"
          })}
        </script>
      </Head>
      <div className="relative mt-12 md:mt-24">
        <div className="flex justify-between items-center mb-8 pl-8 pr-8">
          <div>
            <label className="text-white mr-2">Genre:</label>
            <select
              className="bg-black text-white p-2 rounded transition-all duration-300 ease-in-out hover:bg-gray-800 focus:outline-none"
              value={selectedGenre || ""}
              onChange={(e) => setSelectedGenre(e.target.value ? parseInt(e.target.value) : null)}
            >
              <option value="">All</option>
              {genres.map((genre) => (
                <option key={genre.id} value={genre.id}>
                  {genre.name}
                </option>
              ))}
            </select>
          </div>
          <div>
            <label className="text-white mr-2">Year:</label>
            <select
              className="bg-black text-white p-2 rounded transition-all duration-300 ease-in-out hover:bg-gray-800 focus:outline-none"
              value={selectedYear || ""}
              onChange={(e) => setSelectedYear(e.target.value || null)}
            >
              <option value="">All</option>
              {years.map((year) => (
                <option key={year} value={year}>
                  {year}
                </option>
              ))}
            </select>
          </div>
        </div>
        <div key={fadeKey} className="grid grid-cols-3 sm:grid-cols-3 md:grid-cols-5 lg:grid-cols-8 gap-4 px-4 animate-fadeIn">
          {latestMovies.map((movie) => {
            const slug = `${movie.id}-${movie.title}`.toLowerCase().replace(/ /g, '-');
            const url = `/watch/movie/?slug=${slug}`;

            return (
              <Link key={movie.id} href={url} passHref>
                <div
                  className="relative bg-cover bg-center rounded-lg overflow-hidden shadow-lg hover:scale-105 transition-transform duration-300 ease-in-out cursor-pointer"
                  style={{
                    backgroundImage: `url(https://image.tmdb.org/t/p/w500${movie.poster_path})`,
                    paddingBottom: '150%', 
                  }}
                >
                  <div className="absolute top-2 right-2 bg-black bg-opacity-70 text-yellow-400 text-xs px-2 py-1 rounded flex items-center">
                    <FaStar className="mr-1" />
                    {movie.vote_average.toFixed(1)}
                  </div>
                </div>
              </Link>
            );
          })}
        </div>

        {/* Pagination */}
        {renderPagination()}

        {/* Spacer for Mobile Navbar */}
        <div className="md:hidden h-24"></div>
        {/* Spacer for PC */}
        <div className="hidden md:block h-12"></div>
        <div className="w-full my-4">
          <AdSense />
        </div>
      </div>
    </>
  );
};

export default function MoviesPageWrapper() {
  return (
    <Suspense fallback={<div>Loading...</div>}>
      <Movies />
    </Suspense>
  );
}
